#include "RNAParameters.h"

using std::string;
using std::vector;
using std::cout;
//#define space(S) calloc(1,(S))

extern "C" {
  #include "fold.h"
  #include "fold_vars.h"
  #include "utils.h"
 #include "pair_mat.h"
extern int energy_of_struct_pt (char *string, short * ptable, short *s, short *s1);
  //extern double energy_of_struct (char *string, char * struc);
  //extern short * S;
  //extern short * S1;
  //extern short * pair_table;

  #include "energy_const.h"
  #include "energy_par.h"
}

static short * S;
static short * S1;
static short * pair_table;

bool fast=true;//false;

void RNAParameters::InitializeViennaRNA(std::string sequence){
  if(fast){
    int length=sequence.size();
    S  = new short [length+2];
    S1  = new short [length+2];//NULL;//new short [length+2];
    pair_table =  new short [length+2];
    // pair_table = make_pair_table(const_cast<char*>(std::string(length,'.').c_str()));
    //(short *) MG_space(sizeof(short)*(length+1));
    MakePairTable(const_cast<char*>(std::string(length,'.').c_str()));
    make_pair_matrix();
    S[0] = S1[0] = pair_table[0]=length;
    for (int i=0; i< length; i++) {
      S[i+1] = encode_char(sequence[i]);
      S1[i+1] = alias[S[i+1]];
    } 
  
    initialize_fold(length);
    update_fold_params();
  }
}

void RNAParameters::CleanUpViennaRNA(){
  if(fast){
    free(pair_table);
    free(S); free(S1);
  }
}


double RNAParameters::Evaluate(vector<std::pair<int,int> > node){
  return Evaluate(RNA,node).first;
  /*
     string s=Structure(RNA.size(),node);
     double energy=energy_of_struct(const_cast<char*>(RNA.c_str()),const_cast<char*>(s.c_str()));
     return energy;
  */
}   

std::pair<double,std::string>  RNAParameters::Evaluate(std::string sequence,std::string s){
//double RNAParameters::Evaluate(string sequence,vector<std::pair<int,int> > node){
  bool debug=false;//true;
  double energy;

  if(fast){
     //     char x=s[1000*s.size()];
     //double energy=energy_of_struct(const_cast<char*>(sequence.c_str()),const_cast<char*>(s.c_str()));
    // if(debug) cout<<"pair table "<<energy<<std::endl;
     pair_table = make_pair_table(const_cast<char*>(s.c_str()));
     //     if(debug) cout<<"energy of struct "<<energy<<std::endl;
     energy=energy_of_struct_pt(const_cast<char*>(sequence.c_str()), pair_table, S, S1)/100.;
     free(pair_table);
      if(debug) cout<<"energy "<<energy<<std::endl;
      if(debug) cout<<"Evaluate returns"<<std::endl;
     //return energy;
  }
  else {
    energy=energy_of_struct(const_cast<char*>(sequence.c_str()),const_cast<char*>(s.c_str()));
  }
  return std::make_pair(energy,s);
}   


std::pair<double,std::string>  RNAParameters::Evaluate(std::string sequence,std::vector<std::pair<int,int> > node){
  std::string s=Structure(sequence.size(),node);
     if(debug){
       cout<<"RNAParameters::Evaluate"<<std::endl;
       for(size_t i=0;i<node.size();i++) cout<<Print(node[i])+" ";
       cout<<std::endl;
     }
     if(debug) cout<<s<<std::endl;

  return Evaluate(sequence,s);
}


 
// void RNAParameters::EncodeSeq() {
//   unsigned int i,l;

//   l = RNA.size();
//   S = (short *) space(sizeof(short)*(l+1));
//   S1= (short *) space(sizeof(short)*(l+1));
//   /* S1 exists only for the special X K and I bases and energy_set!=0 */
//   S[0] = S1[0] = (short) l;
  
//   for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
//     S[i]= (short) encode_char(toupper(RNA[i-1]));
//     S1[i] = alias[S[i]];   /* for mismatches of nostandard bases */
//   }
// }




    /** Creates a new instance of RNAParameters */
RNAParameters::RNAParameters(string RNA,string native,int loopsize,int initial_stacksize,int final_stacksize,int reference_size,int n_divisions) {
      this->loopsize=loopsize;
      this->initial_stacksize=initial_stacksize;
      this->stacksize=initial_stacksize;
      this->final_stacksize=final_stacksize;
      this->n_divisions=n_divisions;
      this->RNA=RNA;


      char *sequence  = new char[RNA.size()+1];
      char *structure = new char[RNA.size()+1];
      strcpy(sequence, RNA.c_str());
      this->mfe = fold(sequence, structure);
      this->mfe_structure=std::string(structure);

      this->native=native;
      randomcoil=string(RNA.size(),'.');
      this->reference_size=reference_size;
      CreateContacts();   
      //      EncodeSeq();  
   }



void RNAParameters::AdjustStacksize(int current_iteration,int total_iterations){

  int total_degrade=initial_stacksize-final_stacksize+1;
  float percentage=(float)(current_iteration-1)/(float)total_iterations;
  float current_degrade=total_degrade*percentage;
  //cout<<"current degrade: "+Str(current_degrade)+"\n";
  int degrade=(int)floor(current_degrade);
  //cout<<"degrade: "+Str(degrade)+"\n";
  int new_stacksize=initial_stacksize-degrade;
  //std::cout<<"updated initial stacksize from"+Str(stacksize)+" to "+Str(new_stacksize)+"\n";
  stacksize=new_stacksize;

}

    bool RNAParameters::CanPair(int i,int j){ return CanPair(RNA[i-1],RNA[j-1]);}
   
       
    bool RNAParameters::CanPair(char base1,char base2){
      //cout<<"Called CanPair with"<<base1<<" and "<<base2<<"\n";
      if(base1=='A' && base2=='U') return true;
      if(base1=='U' && base2=='A') return true;
      if(base1=='G' && base2=='C') return true;
      if(base1=='C' && base2=='G') return true;
      if(base1=='G' && base2=='U') return true;
      if(base1=='U' && base2=='G') return true;
      return false;
   }
        


vector<vector<std::pair<int,int> > > RNAParameters::ConformationToStacks(vector<std::pair<int,int> > node){
  vector<vector<std::pair<int,int> > > stacks= vector<vector<std::pair<int,int> > >();
  //vector<std::pair<int,int> > contacts=Contacts(node);
  sort(node.begin(),node.end());
  //std::pair<int,int> prev=node.front();
  vector<std::pair<int,int> > stack(node.begin(),node.begin()+1);//=vector<std::pair<int,int> >();
  //  stack.push_back(prev);
  //bool added=false;
  for(size_t i=1;i<node.size();i++){
    bool added=false;
    if(node[i].first==stack.back().first+1 && node[i].second==stack.back().second-1) {
      added=true;
      stack.push_back(std::pair<int,int>(stack.back().first+1,stack.back().second-1));
    }
    else if(node[i].first==stack.back().first+2 && node[i].second==stack.back().second-1) {
      added=true;
      stack.push_back(std::pair<int,int>(stack.back().first+2,stack.back().second-1));
    }
    else if(node[i].first==stack.back().first+1 && node[i].second==stack.back().second-2) {
      added=true;
      stack.push_back(std::pair<int,int>(stack.back().first+1,stack.back().second-2));
    }
    if((int)stack.size()>=stacksize && (!added || i==node.size()-1) ) {
      stacks.push_back(stack);
      stack.clear();
      stack.push_back(node[i]);
    }
    if(!added) {
      stack.clear();
      stack.push_back(node[i]);
    }
  }
  return stacks;
}

/*
vector<vector<std::pair<int,int> > > RNAParameters::ConformationToStacks(vector<int> node){
  vector<vector<std::pair<int,int> > > stacks= vector<vector<std::pair<int,int> > >();
  vector<std::pair<int,int> > contacts=Contacts(node);
  sort(contacts.begin(),contacts.end());
  std::pair<int,int> prev=contacts.front();
  int size=1;
  for(int i=1;i<contacts.size();i++){
    if(contacts[i].first==prev.first+1 && contacts[i].second==prev.second-1) {
      size++;
      //in this case the current pair needs to be included
      if(size>=stacksize && i==contacts.size()-1) 	stacks.push_back(vector<std::pair<int,int> > (contacts.begin()+i+1-size,contacts.begin()+i+1));
    }
    else {
      //current pair is excluded
      if(size>=stacksize)                               stacks.push_back(vector<std::pair<int,int> > (contacts.begin()+i-size,contacts.begin()+i)); 
      size=1;
    }
    prev=contacts[i];
  }
  return stacks;
}
*/

   
    
// int RNAParameters::ContactIndex(std::pair<int,int> p){     
//      for(size_t i=0;i<contacts.size();i++){
//          if(contacts[i].first==p.first && contacts[i].second==p.second) return i;    
//       }

//    }
    

   std::pair<int,int> RNAParameters::Contact(int idx){return contacts[idx];}
    
   /**
    *   Uses the act that contacs are sorted to returns the base pairs sorted.
    */
   

vector<std::pair<int,int> > RNAParameters::Contacts(vector<int> idx){
       vector<std::pair<int,int> > contacts =vector<std::pair<int,int> >();
       sort(idx.begin(),idx.end());
       
       //for(int i=0;i<idx.size();i++){
	 for(vector<int>::iterator it=idx.begin();it!=idx.end();it++) {
	 
	 //std::pair<int,int> contact=Contact(idx[i]);

	 //contacts.push_back(contact);
	 	 contacts.push_back(Contact(*it));
       }
       return contacts;
   }
    
    void  RNAParameters::CreateContacts(){
      //cout<<"create contacts\n";
      contacts=vector<std::pair<int,int> >();
      int count=0;
      for(size_t i=1;i<=RNA.length();i++) { 
         for(size_t j=i+loopsize+1;j<=RNA.length();j++){
	   if(!CanPair((int)i,(int)j)) continue;
            contacts.push_back(std::pair<int,int>(i,j));    
            ////cout<<count+": ("+Str(i)+","+Str(j)+")\n";
	    count++;
         }
      }
   }
      
    
  
    
   
vector<std::pair<int,int> > RNAParameters::CreateStack(){  
  return CreateStack(1,RNA.size(),reference_size);
}  
        
vector<std::pair<int,int> > RNAParameters::CreateStack(int from,int to,int size){    

            //cout<<" CreateStack called with from="+Str(from)+" and to="+Str(to)+" and size="+Str(size)+"\n";
            if(size==0) return vector<std::pair<int,int> >();
            //cout<<"create stack of size "+Str(size)+"\n";
	    int count=0;
            while(count++<to-from){
                //pick a random base
	      /*
		Turn off subdivisions until the need for them becomes apparent again
	      */
   	        //int section_size=RNA.size()/n_divisions;
                //int offset=(rand()%n_divisions)*section_size;
                //int left=offset+(rand()%section_size);

		int left=from+size-1+(rand()%(to-from));
                if(left<from+size) continue;
                
                //xxx ASK XTOF what number should be an upper bound for a hairpin
                int hairpin_size=6;
		  int loop=loopsize+rand()%(hairpin_size+1-loopsize);
                int right=left+loop+1;
                int max_right=to-size;



                //try to construct a hairpin from the inside out
                //there should enough bases after right to build a stack
                do right++;
                while(right<=max_right && !CanPair(left,right) );
                if(right>max_right) continue;
                
                vector<std::pair<int,int> > stack=vector<std::pair<int,int> >();
                //edge_left is the idx of the base closest to 5'in the stack constructed so far
                int edge_left=left;
                int edge_right=right;
		while(edge_left>=from && edge_right<=to){
		  if(CanPair(edge_left,edge_right)){
                    stack.push_back(std::pair<int,int>(edge_left,edge_right));
		    edge_left--;
                    edge_right++;
		  }
                  else if( edge_left>from && CanPair(edge_left-1,edge_right) ){
		    stack.push_back(std::pair<int,int>(edge_left-1,edge_right));
		     edge_left-=2;
                     edge_right++;
		  }
                  else if( edge_right<to && CanPair(edge_left,edge_right+1)){
		    stack.push_back(std::pair<int,int>(edge_left,edge_right+1));
    		     edge_left--;
                     edge_right+=2;
		  }
		  else break;
		}
                //edge_left/edge_right are pushed 1 beyond whatis currently known to pair. These values therefore are too high when the loop exits.
                //can't build a stack of sufficient size. try a new random contact.
                if(left-edge_left<size) continue;         
                // cout<<"Found a stack where left="+Str(left)+" and edge_left="+Str(edge_left)+" and size="+Str(size)+"\n";
                sort(stack.begin(),stack.end());
                //  cout<<"Found a stack of size "+Str(stack.size())+"\n";
		stack=HighestSubStack(stack);

		if(Evaluate(stack)>=0) continue;
                return stack;
		/*
                vector<int> ret=vector<int>();
                //add all std::pairs of contacts with left values from outer_left to outer right;
                for(int i=0;i<stack.size();i++){
                    int idx=ContactIndex(stack[i]);
		    ret.push_back(idx);
                }
                sort(ret.begin(),ret.end());

		//ensure the energy contribution is negative
		if(Evaluate(ret)>=0) continue;
                //cout<<"print contacts created by CreateStack\n";
                //cout<<Print(ret)+"\n";
                return ret;
		*/
            }   
	    return vector<std::pair<int,int> >();
        }        
     


/*
  Can a stem be extended further than the terminal basepair?
*/

void 	RNAParameters::ExtendStackMaximallyIn(std::vector<std::pair<int,int> >& stem){
  sort(stem.begin(),stem.end());
  int next_first=stem.back().first+1;
  int next_second=stem.back().second-1;
  while(next_second-next_first>loopsize){
    //idx of terminal basepair in RNA, starting at 0
    if(!CanPair(RNA[next_first-1],RNA[next_second-1])) break;
    stem.push_back(std::pair<int,int>(next_first,next_second));
    next_first++;
    next_second--;
  }
}

void 	RNAParameters::ExtendStackMaximallyOut(std::vector<std::pair<int,int> >& stem){
  sort(stem.begin(),stem.end());
  int next_first=stem.front().first-1;
  int next_second=stem.front().second+1;
  while(true){//next_second-next_first>loopsize){
    //idx of terminal basepair in RNA, starting at 0
    if(next_first-1<=0 || next_second+1>(int)RNA.size() || !CanPair(RNA[next_first-1],RNA[next_second-1])) break;
    stem.push_back(std::pair<int,int>(next_first,next_second));
    next_first--;
    next_second++;
  }
}


std::vector<std::vector<std::pair<int,int> > > 	RNAParameters::FindSuperStems(std::vector<std::pair<int,int> > stem){
  std::vector<std::vector<std::pair<int,int> > > super_stems=std::vector<std::vector<std::pair<int,int> > >();
  super_stems.push_back(stem);
  sort(stem.begin(),stem.end());
  int next_first=stem.back().first+1;
  int next_second=stem.back().second-1;
  while(next_second-next_first>loopsize){
    //idx of terminal basepair in RNA, starting at 0
    if(!CanPair(RNA[next_first-1],RNA[next_second-1])) break;
    stem.push_back(std::pair<int,int>(next_first,next_second));
    super_stems.push_back(stem);
    next_first++;
    next_second--;
  }
  return super_stems;
}

/*
  Checks whether the pairs (s1[0],s2[n]) .....(s1[n],s2[0]) can form basepairs

 */


bool RNAParameters::CanPair(std::string s1,std::string s2){
  //called CanPair for GGA and CCC

  //cout<<"called CanPair for "+s1+" and "+s2+"\n";
  if(s1.size()!=s2.size()) return false;
  int size=s1.size();
  for(int i=0;i<size;i++){
    if(!CanPair(s1[i],s2[size-1-i])) return false;
  }
  return true;

}


/*
NOTE: THis list is made only once and needs to include all stems that ever become relevant.
It therefore sets the cutoff for size at final_stacksize.

*/
std::vector<std::vector<std::pair<int,int> > > RNAParameters::EnumerateStems(){
  // bool debug=true;
  // return EnumerateStemsWithSubstems();
  if(debug)cout<<"Enumerate stems called\n";
  std::vector<std::vector<std::pair<int,int> > > stemlist=std::vector<std::vector<std::pair<int,int> > >();
  if(debug)cout<<RNA<<"\n";
  int min_stem_length=2*final_stacksize+loopsize;
  int latest_stem_start=RNA.size()-min_stem_length+1;
  for(int i=0;i<latest_stem_start;i++){
    if(debug) cout<<"i is"<<i<<"\n";  
  //find initial_stacksize consecutive base pairs. Then find all stems that canbe built from them. Then extend each stem to its maximum size.

    //consdier whether we allow bulges...
    string stem_five_prime=RNA.substr(i,final_stacksize);
    //go through each final_stacksize-tupel in RNA and check whether each nucleotide can pair with the corrpsonding element in stem_five_prime
    for(size_t j=i+min_stem_length-1;j<RNA.size();j++){
      if(debug) cout<<"j is"<<j<<"\n";  
      string stem_three_prime=RNA.substr(j-final_stacksize+1,final_stacksize);
      if(debug) cout<<"Can "+stem_five_prime+" and "+stem_three_prime+" pair?\n";
      if(CanPair(stem_five_prime,stem_three_prime)){
        if(debug) cout<<"Yes\n";
	//try to expand the stem
	std::vector<std::pair<int,int> > stem=std::vector<std::pair<int,int> >();
	for(int k=0;k<final_stacksize;k++) {
	  //+1s are to accomodate the fasct that basepairs are counted from 1, but array indices are counted from 0.
	  stem.push_back(std::pair<int,int>(i+1+k,j+1-k));
	}
	if(debug) cout<<"original:"+Print(stem);
	ExtendStackMaximallyIn(stem);
	ExtendStackMaximallyOut(stem);
	if(debug) cout<<"extended:"+Print(stem);
	if(Evaluate(stem)<0) {
          sort(stem.begin(),stem.end());
          //add stems that are new
          if(find(stemlist.begin(),stemlist.end(),stem)==stemlist.end()) stemlist.push_back(stem);
	  }
	//else cout<<"energy too low:"+Str(Evaluate(stem))+"\n";
      }
    }
  }
  if(debug) cout<<"enumerate stems returns\n";
  return stemlist;
 
}

std::vector<std::vector<std::pair<int,int> > > RNAParameters::EnumerateStemsWithSubstems(){


  if(debug)cout<<"Enumerate stems called\n";
  std::vector<std::vector<std::pair<int,int> > > stemlist=std::vector<std::vector<std::pair<int,int> > >();
  if(debug)cout<<RNA<<"\n";
  int min_stem_length=2*final_stacksize+loopsize;
  int latest_stem_start=RNA.size()-min_stem_length+1;
  for(int i=0;i<latest_stem_start;i++){
    if(debug) cout<<"i is"<<i<<"\n";  
  //find initial_stacksize consecutive base pairs. Then find all stems that canbe built from them. Then extend each stem to its maximum size.

    //consdier whether we allow bulges...
    string stem_five_prime=RNA.substr(i,final_stacksize);
    //go through each final_stacksize-tupel in RNA and check whether each nucleotide can pair with the corrpsonding element in stem_five_prime
    for(size_t j=i+min_stem_length-1;j<RNA.size();j++){
      if(debug) cout<<"j is"<<j<<"\n";  
      string stem_three_prime=RNA.substr(j-final_stacksize+1,final_stacksize);
      if(debug) cout<<"Can "+stem_five_prime+" and "+stem_three_prime+" pair?\n";
      if(CanPair(stem_five_prime,stem_three_prime)){
        if(debug) cout<<"Yes\n";
	//try to expand the stem
	std::vector<std::pair<int,int> > stem=std::vector<std::pair<int,int> >();
	for(int k=0;k<final_stacksize;k++) {
	  //+1s are to accomodate the fasct that basepairs are counted from 1, but array indices are counted from 0.
	  stem.push_back(std::pair<int,int>(i+1+k,j+1-k));
	}
        if(debug) cout<<"original stem:\n";
	if(debug) cout<<Print(stem);
	std::vector<std::vector<std::pair<int,int> > > super_stems=FindSuperStems(stem);
        if(debug) cout<<"super_stems.size():"+Str((int)super_stems.size())+"\n";
	//add super_stem, if it has negative energy and is not yet in stemlist
	for(size_t l=0;l<super_stems.size();l++){
	  if(Evaluate(super_stems[l])<0 ){
            if(debug) cout<<"super_stem energy:"+Str(Evaluate(super_stems[l]))+"\n";
	    if(find(stemlist.begin(),stemlist.end(),super_stems[l])==stemlist.end() ) {
              if(debug) cout<<"add super_stem:\n";
	      if(debug) cout<<Print(super_stems[l]);
	      stemlist.push_back(super_stems[l]);
	    }
	  }
	}
      }
    }
  }
  //if(debug) 
  //cout<<"return stemlist of size "+Str(stemlist.size())+"\n";
  return stemlist;
}



std::vector<int>
 RNAParameters::MakePairTable(std::string structure)
{
  short* pt = NULL;
 
  pt = make_pair_table(structure.c_str());
  std::vector<int> tbl=std::vector<int> ();
  tbl.reserve(pt[0]);
  for(int i = 1; i<=pt[0]; i++) {
    int value = (pt[i]==0) ? -1 : static_cast<int>(pt[i]-1);
    tbl.push_back(value);
  }
  free(pt);
  return (tbl);
}

std::vector<std::pair<int,int> >  RNAParameters::MakeBasePairList (std::string structure){
  std::vector<int> pTbl = MakePairTable(structure); 
  std::vector<std::pair<int,int> > pLst = std::vector<std::pair<int,int> > ();
  pLst.reserve(pTbl.size());
  int i = 0;
  for (std::vector<int>::iterator j = pTbl.begin();
       j != pTbl.end();
       j++, i++) {

    // skip unpaired positions and entries with pTbl[i] < i
    if (*j == -1 || *j < i) continue;

    pLst.push_back(std::make_pair(i, *j));
  }

  return (pLst);
}
   





bool SequenceCompare(std::pair<int,int> s1,std::pair<int,int> s2){
  return (s1.second-s1.first<s2.second-s2.first);

}

/**
   What is this for?
*/

std::pair<int,int> RNAParameters::FindMaximalUnpairedSubsequence(vector<std::pair<int,int> > node){
  //vector<std::pair<int,int> > contacts=Contacts(node);
  vector<std::pair<int,int> > results=vector<std::pair<int,int> > ();
  vector<std::pair<int,int> > sequences=vector<std::pair<int,int> > (); 
  sequences.push_back(std::pair<int,int>(1,RNA.size()));


  while(sequences.size()!=0){
    std::pair<int,int> p=sequences.back();
    sequences.pop_back();
    //find the first contact that falls into the subrange.
    int idx1=-1;
    for(size_t i=0;i<node.size();i++){
      if(IntervalContainsBasePair(p,node[i])){
	   idx1=i;
	   break;
      }
    }
    if(idx1==-1){
      results.push_back(p);
      continue;
    }
    //since the contacts are sorted, there are no more stacks in [p.first,contactsp[idx1].first];
    results.push_back(std::pair<int,int>(p.first,node[idx1].first-1) );
    int idx2=idx1;
    while(1+node[idx2].first==node[idx2+1].first) idx2++;
    sequences.push_back(std::pair<int,int>(node[idx1].second+1,p.second));  
    sequences.push_back(std::pair<int,int>(node[idx2].first+1, node[idx2].second-1));

  }
  //sort sequences in ascending order
  sort(results.begin(),results.end(),SequenceCompare);
  return results.back();
}

        
        
   /*
  Assumes as usual that the elements in the stack are sorted by left base pairs
*/

vector<std::pair<int,int> > RNAParameters::HighestSubStack(vector<std::pair<int,int> > stack){
  //remove base pairs from inside as long as it improves the score
  //inside= back of stack, since left element of pair comes first in ordering.
  while((int)stack.size()>stacksize && Evaluate(stack)>=Evaluate(vector<std::pair<int,int> >(stack.begin(),stack.end()-1)) ) stack.pop_back();
  while((int)stack.size()>stacksize && Evaluate(stack)>=Evaluate(vector<std::pair<int,int> >(stack.begin()+1,stack.end())) ) stack.erase(stack.begin());
  return stack;

  }
/*
vector<std::pair<int,int> > RNAParameters::HighestSubStack(vector<std::pair<int,int> > stack){

  cout<<"Highest substack called with stack "+Print(stack)+"\n";
  if(stack.size()==stacksize) return stack;
  cout<<"min size for stacks is "+Str(stacksize)+"\n";
  cout<<"size stack for which a substack is wanted is  "+Str(stack.size())+"\n";


  vector<double> energies=vector<double>();
  for(int i=0;i<stack.size()-stacksize-1;i++){
    cout<<"i is "+Str(i)+"\n";
    vector<std::pair<int,int> > substack(stack.begin()+i,stack.end());
    energies.push_back(Evaluate(substack));
    cout<<Print(substack)+" "+Str(energies.back())+"\n";
  }
  
  int idx=min_element(energies.begin(),energies.end())-energies.begin();
  cout<<"index of minmal element: "+Str(idx)+"\n";
  cout<<"returning substack:"+Print(vector<std::pair<int,int> > (stack.begin()+idx,stack.end()))+"\n";
  return vector<std::pair<int,int> > (stack.begin()+idx,stack.end());
}
*/

bool RNAParameters::IntervalContainsBasePair(std::pair<int,int> interval,std::pair<int,int> basepair){ 
  return (interval.first<=basepair.first && interval.second>=basepair.second);
}
     
    



int RNAParameters::NextPairedBaseLeft(std::vector<std::pair<int,int> > node,int start){
  string structure=Structure(RNA.size(),node);
  for(int i=start-1;i>0;i--){
    if(structure[i-1]!='.') return i; 
  }
  //if the first base is paired, return 1
  //default: not even the first base is paired
  return 0;
}


int RNAParameters::NextPairedBaseRight(std::vector<std::pair<int,int> > node,int start){
  string structure=Structure(RNA.size(),node);
  for(size_t i=start+1;i<=RNA.size();i++){
    if(structure[i-1]!='.') return i; 
  }
  //if the last base is paired, return RNA.size()
  //default: not even the last base is paired
  return RNA.size()+1;
}


   
string RNAParameters::Print(int i,int j){ return "("+Str(i)+","+Str(j)+")";}
 
string RNAParameters::Print(std::pair<int,int> p){ return Print(p.first,p.second);}

/*
string RNAParameters::Print(vector<int> node){ 
  string s=string();
  for(int i=0;i<node.size();i++){
    s+=Print(Contact(node[i]))+"\n";
    //s+=node[i]+"\n";
  }
  return s;
}
*/

/*
string RNAParameters::PrintPair(int i,int j){ return "("+Str(i)+","+Str(j)+")";}
*/

string RNAParameters::Print(std::vector<std::vector<std::pair<int,int> > > vvpair){ 
   string s;
  for(size_t i=0;i<vvpair.size();i++){
    s+=Str((int)i)+":"+Print(vvpair[i]);
  }
  return s+"\n";

}



string RNAParameters::Print(std::vector<std::pair<int,int> > vpair){ 
  string s;
  for(size_t i=0;i<vpair.size();i++){
    s+=Print(vpair[i]);
  }
  return s+"\n";
}



std::string RNAParameters::Structure(int length, std::vector<std::pair<int,int> > pairs){
  //cout<<"RNAParameters::Structure for sequence of length "+Str(length)+"\n";

    std::string s=std::string(length,'.');
  for(size_t i=0;i<pairs.size();i++) {
    //std::cout<<"structure was \n "+s+"\n";
    std::pair<int,int> p=pairs[i];
    //std::cout<<"add pair ("+Str(pairs[i].first)+","+Str(pairs[i].second)+")\n";
    s[p.first-1]='(';
    s[p.second-1]=')';
    //std::cout<<"structure becomes\n "+s+"\n";
  }
  //std::cout<<" return structure \n"+s+"\n";
  return s;


}

