#ifndef RNA_PARAMETERS_H
#define RNA_PARAMETERS_H 1


#include <math.h>
#include <cstdlib>
#include <iostream>
#include <string>
#include <utility>
#include <vector>
//#include "vienna.h"
#include "utl.h"

 class RNAParameters {
    
 public:
   //   short *S;
   
   //    short *S1;
    
    double mfe;
    std::string mfe_structure;

    std::vector<std::pair<int,int> > contacts;
    
    int loopsize;
    
    int stacksize;

    int initial_stacksize;
    

    int final_stacksize;
    //number of sections to divide the RNA into for the purpose of finding stacks
    int n_divisions;
    
    std::string randomcoil;
    
    std::string RNA;

    std::string native;
    
    int reference_size;
    
    static const bool debug=false;

    RNAParameters(std::string RNA,std::string native,int loopsize,int initial_stacksize,int final_stacksize,int reference_size,int n_divisions);
  
    static void InitializeViennaRNA(std::string sequence);

    static void CleanUpViennaRNA();
    /**
       Used as a cop-out so RNAPredict can call the Print memberfunction for stacks. Needs a global variable.
    */
    // RNAParameters();


    void AdjustStacksize(int current_iteration,int total_iterations);
    /**
       Initial Conformation
    */

    void CreateContacts();


    /**
       Create Stem/Stack

    */

     std::vector<std::pair<int,int> >  CreateStack();

     std::vector<std::pair<int,int> >  CreateStack(int from,int to,int size);

    int ContactIndex(std::pair<int,int> p);

    std::pair<int,int> Contact(int idx);

    std::vector<std::pair<int,int> > Contacts(std::vector<int> idx);


     std::vector<std::vector<std::pair<int,int> > > EnumerateStems();
     std::vector<std::vector<std::pair<int,int> > > EnumerateStemsWithSubstems();
    /**
       Called by CreateStack, to ensure that not the longest stack, but the sub-stack with the best free energy is selected.

    */    

    std::vector<std::pair<int,int> > HighestSubStack(std::vector<std::pair<int,int> > stack);

    bool CanPair(int i,int j);


    bool CanPair(char base1,char base2);

    bool CanPair(std::string s1,std::string s2);

    void EncodeSeq();


    static std::pair<double,std::string>  Evaluate(std::string sequence,std::string s);

    static std::pair<double,std::string> Evaluate(std::string sequence,std::vector<std::pair<int,int> > node);

    double Evaluate(std::vector<std::pair<int,int> > node);


    void ExtendStackMaximallyIn(std::vector<std::pair<int,int> >& stem);
    void ExtendStackMaximallyOut(std::vector<std::pair<int,int> >& stem);

    std::vector<std::vector<std::pair<int,int> > > FindSuperStems(std::vector<std::pair<int,int> > stem);
    /**
       Translate a vector of base pairs into a bracket-dot expression

    */
 static std::string Structure(int length, std::vector<int> pairs);

 static std::string Structure(int length, std::vector<std::pair<int,int> > pairs);


   /**
     For finding maximal unpaired subsequences.
   
*/

    static bool IntervalContainsBasePair(std::pair<int,int> interval,std::pair<int,int> basepair);

    /**
       When Swarm tries to find a new stem/stack
    */


    std::pair<int,int> FindMaximalUnpairedSubsequence(std::vector<std::pair<int,int> > node);


    /**
       Lots of Print functions
    */

    static std::string Print(int i,int j);
 
    static std::string Print(std::pair<int,int> p);

    //static std::string Print(std::vector<int> node);


    //std::string PrintPair(int i,int j);

    static std::string Print(std::vector<std::pair<int,int> > vpair);


    static std::string Print(std::vector<std::vector<std::pair<int,int> > > vpair);
    /**
       Split a set of base pairs into stacks
    */

    std::vector<std::vector<std::pair<int,int> > > ConformationToStacks(std::vector<std::pair<int,int> > node);

    int NextPairedBaseLeft(std::vector<std::pair<int,int> > node,int start);

    int NextPairedBaseRight(std::vector<std::pair<int,int> > node,int start);

static std::vector<int> MakePairTable(std::string structure);
static std::vector<std::pair<int,int> > MakeBasePairList (std::string structure);

 };

#endif
